@@ -21,7 +21,8 @@ import (
 	"fmt"
 	"sort"
 	"strings"
-	"text/template"
+
+	"github.com/google/safetext/yamltemplate"
 
 	"sigs.k8s.io/kind/pkg/errors"
 
@@ -56,7 +57,7 @@ type ConfigData struct {
 	// The Token for TLS bootstrap
 	Token string
 
-	// KubeProxyMode defines the kube-proxy mode between iptables or ipvs
+	// KubeProxyMode defines the kube-proxy mode between iptables, ipvs or nftables
 	KubeProxyMode string
 	// The subnet used for pods
 	PodSubnet string
@@ -78,10 +79,6 @@ type ConfigData struct {
 	// RootlessProvider is true if kind is running with rootless mode
 	RootlessProvider bool
 
-	// DisableLocalStorageCapacityIsolation is typically set true based on RootlessProvider
-	// based on the Kubernetes version, if true kubelet localStorageCapacityIsolation is set false
-	DisableLocalStorageCapacityIsolation bool
-
 	// DerivedConfigData contains fields computed from the other fields for use
 	// in the config templates and should only be populated by calling Derive()
 	DerivedConfigData
@@ -94,8 +91,8 @@ type DerivedConfigData struct {
 	AdvertiseAddress string
 	// DockerStableTag is automatically derived from KubernetesVersion
 	DockerStableTag string
-	// SortedFeatureGateKeys allows us to iterate FeatureGates deterministically
-	SortedFeatureGateKeys []string
+	// SortedFeatureGates allows us to iterate FeatureGates deterministically
+	SortedFeatureGates []FeatureGate
 	// FeatureGatesString is of the form `Foo=true,Baz=false`
 	FeatureGatesString string
 	// RuntimeConfigString is of the form `Foo=true,Baz=false`
@@ -106,6 +103,15 @@ type DerivedConfigData struct {
 	IPv6 bool
 	// kubelet cgroup driver, based on kubernetes version
 	CgroupDriver string
+	// JoinSkipPhases are the skipPhases values for the JoinConfiguration.
+	JoinSkipPhases []string
+	// InitSkipPhases are the skipPhases values for the InitConfiguration.
+	InitSkipPhases []string
+}
+
+type FeatureGate struct {
+	Name  string
+	Value bool
 }
 
 // Derive automatically derives DockerStableTag if not specified
@@ -130,13 +136,17 @@ func (c *ConfigData) Derive() {
 		featureGateKeys = append(featureGateKeys, k)
 	}
 	sort.Strings(featureGateKeys)
-	c.SortedFeatureGateKeys = featureGateKeys
 
 	// create a sorted key=value,... string of FeatureGates
-	var featureGates []string
+	c.SortedFeatureGates = make([]FeatureGate, 0, len(c.FeatureGates))
+	featureGates := make([]string, 0, len(c.FeatureGates))
 	for _, k := range featureGateKeys {
 		v := c.FeatureGates[k]
 		featureGates = append(featureGates, fmt.Sprintf("%s=%t", k, v))
+		c.SortedFeatureGates = append(c.SortedFeatureGates, FeatureGate{
+			Name:  k,
+			Value: v,
+		})
 	}
 	c.FeatureGatesString = strings.Join(featureGates, ",")
 
@@ -156,6 +166,15 @@ func (c *ConfigData) Derive() {
 		runtimeConfig = append(runtimeConfig, fmt.Sprintf("%s=%s", k, v))
 	}
 	c.RuntimeConfigString = strings.Join(runtimeConfig, ",")
+
+	// Skip preflight to avoid pulling images.
+	// Kind pre-pulls images and preflight may conflict with that.
+	// requires kubeadm 1.22+
+	c.JoinSkipPhases = []string{"preflight"}
+	c.InitSkipPhases = []string{"preflight"}
+	if c.KubeProxyMode == string(config.NoneProxyMode) {
+		c.InitSkipPhases = append(c.InitSkipPhases, "addon/kube-proxy")
+	}
 }
 
 // See docs for these APIs at:
@@ -173,7 +192,7 @@ kubernetesVersion: {{.KubernetesVersion}}
 clusterName: "{{.ClusterName}}"
 {{ if .KubeadmFeatureGates}}featureGates:
 {{ range $key, $value := .KubeadmFeatureGates }}
-  "{{ $key }}": {{ $value }}
+  "{{ (StructuralData $key) }}": {{ $value }}
 {{end}}{{end}}
 controlPlaneEndpoint: "{{ .ControlPlaneEndpoint }}"
 # on docker for mac we have to expose the api server via port forward,
@@ -272,10 +291,10 @@ evictionHard:
   nodefs.inodesFree: "0%"
   imagefs.available: "0%"
 {{if .FeatureGates}}featureGates:
-{{ range $key := .SortedFeatureGateKeys }}
-  "{{ $key }}": {{ index $.FeatureGates $key }}
+{{ range $index, $gate := .SortedFeatureGates }}
+  "{{ (StructuralData $gate.Name) }}": {{ $gate.Value }}
 {{end}}{{end}}
-{{if ne .KubeProxyMode "None"}}
+{{if ne .KubeProxyMode "none"}}
 ---
 apiVersion: kubeproxy.config.k8s.io/v1alpha1
 kind: KubeProxyConfiguration
@@ -283,8 +302,8 @@ metadata:
   name: config
 mode: "{{ .KubeProxyMode }}"
 {{if .FeatureGates}}featureGates:
-{{ range $key := .SortedFeatureGateKeys }}
-  "{{ $key }}": {{ index $.FeatureGates $key }}
+{{ range $index, $gate := .SortedFeatureGates }}
+  "{{ (StructuralData $gate.Name) }}": {{ $gate.Value }}
 {{end}}{{end}}
 iptables:
   minSyncPeriod: 1s
@@ -292,6 +311,12 @@ conntrack:
 # Skip setting sysctl value "net.netfilter.nf_conntrack_max"
 # It is a global variable that affects other namespaces
   maxPerCore: 0
+# Set sysctl value "net.netfilter.nf_conntrack_tcp_be_liberal"
+# for nftables proxy (theoretically for kernels older than 6.1)
+# xref: https://github.com/kubernetes/kubernetes/issues/117924
+{{if and (eq .KubeProxyMode "nftables") (not .RootlessProvider)}}
+  tcpBeLiberal: true
+{{end}}
 {{if .RootlessProvider}}
 # Skip setting "net.netfilter.nf_conntrack_tcp_timeout_established"
   tcpEstablishedTimeout: 0s
@@ -310,7 +335,7 @@ kubernetesVersion: {{.KubernetesVersion}}
 clusterName: "{{.ClusterName}}"
 {{ if .KubeadmFeatureGates}}featureGates:
 {{ range $key, $value := .KubeadmFeatureGates }}
-  "{{ $key }}": {{ $value }}
+  "{{ (StructuralData $key) }}": {{ $value }}
 {{end}}{{end}}
 controlPlaneEndpoint: "{{ .ControlPlaneEndpoint }}"
 # on docker for mac we have to expose the api server via port forward,
@@ -364,6 +389,12 @@ nodeRegistration:
     node-ip: "{{ .NodeAddress }}"
     provider-id: "kind://{{.NodeProvider}}/{{.ClusterName}}/{{.NodeName}}"
     node-labels: "{{ .NodeLabels }}"
+{{ if .InitSkipPhases -}}
+skipPhases:
+  {{- range $phase := .InitSkipPhases }}
+  - "{{ $phase }}"
+  {{- end }}
+{{- end }}
 ---
 # no-op entry that exists solely so it can be patched
 apiVersion: kubeadm.k8s.io/v1beta3
@@ -387,6 +418,12 @@ discovery:
     apiServerEndpoint: "{{ .ControlPlaneEndpoint }}"
     token: "{{ .Token }}"
     unsafeSkipCAVerification: true
+{{ if .JoinSkipPhases -}}
+skipPhases:
+  {{ range $phase := .JoinSkipPhases -}}
+  - "{{ $phase }}"
+  {{- end }}
+{{- end }}
 ---
 apiVersion: kubelet.config.k8s.io/v1beta1
 kind: KubeletConfiguration
@@ -409,11 +446,10 @@ evictionHard:
   nodefs.inodesFree: "0%"
   imagefs.available: "0%"
 {{if .FeatureGates}}featureGates:
-{{ range $key := .SortedFeatureGateKeys }}
-  "{{ $key }}": {{ index $.FeatureGates $key }}
+{{ range $index, $gate := .SortedFeatureGates }}
+  "{{ (StructuralData $gate.Name) }}": {{ $gate.Value }}
 {{end}}{{end}}
-{{if .DisableLocalStorageCapacityIsolation}}localStorageCapacityIsolation: false{{end}}
-{{if ne .KubeProxyMode "None"}}
+{{if ne .KubeProxyMode "none"}}
 ---
 apiVersion: kubeproxy.config.k8s.io/v1alpha1
 kind: KubeProxyConfiguration
@@ -421,8 +457,8 @@ metadata:
   name: config
 mode: "{{ .KubeProxyMode }}"
 {{if .FeatureGates}}featureGates:
-{{ range $key := .SortedFeatureGateKeys }}
-  "{{ $key }}": {{ index $.FeatureGates $key }}
+{{ range $index, $gate := .SortedFeatureGates }}
+  "{{ (StructuralData $gate.Name) }}": {{ $gate.Value }}
 {{end}}{{end}}
 iptables:
   minSyncPeriod: 1s
@@ -430,6 +466,12 @@ conntrack:
 # Skip setting sysctl value "net.netfilter.nf_conntrack_max"
 # It is a global variable that affects other namespaces
   maxPerCore: 0
+# Set sysctl value "net.netfilter.nf_conntrack_tcp_be_liberal"
+# for nftables proxy (theoretically for kernels older than 6.1)
+# xref: https://github.com/kubernetes/kubernetes/issues/117924
+{{if and (eq .KubeProxyMode "nftables") (not .RootlessProvider)}}
+  tcpBeLiberal: true
+{{end}}
 {{if .RootlessProvider}}
 # Skip setting "net.netfilter.nf_conntrack_tcp_timeout_established"
   tcpEstablishedTimeout: 0s
@@ -458,16 +500,6 @@ func Config(data ConfigData) (config string, err error) {
 			return "", errors.Errorf("version %q is not compatible with rootless provider (hint: kind v0.11.x may work with this version)", ver)
 		}
 		data.FeatureGates["KubeletInUserNamespace"] = true
-
-		// For avoiding err="failed to get rootfs info: failed to get device for dir \"/var/lib/kubelet\": could not find device with major: 0, minor: 41 in cached partitions map"
-		// https://github.com/kubernetes-sigs/kind/issues/2524
-		if ver.LessThan(version.MustParseSemantic("v1.25.0-alpha.3.440+0064010cddfa00")) {
-			// this feature gate was removed in v1.25 and replaced by an opt-out to disable
-			data.FeatureGates["LocalStorageCapacityIsolation"] = false
-		} else {
-			// added in v1.25 https://github.com/kubernetes/kubernetes/pull/111513
-			data.DisableLocalStorageCapacityIsolation = true
-		}
 	}
 
 	// assume the latest API version, then fallback if the k8s version is too low
@@ -476,7 +508,7 @@ func Config(data ConfigData) (config string, err error) {
 		templateSource = ConfigTemplateBetaV2
 	}
 
-	t, err := template.New("kubeadm-config").Parse(templateSource)
+	t, err := yamltemplate.New("kubeadm-config").Parse(templateSource)
 	if err != nil {
 		return "", errors.Wrap(err, "failed to parse config template")
 	}
